/****************************************************************
**
** GameTreeViewer 
**
****************************************************************/
#include <qapplication.h>
#include <qfiledialog.h>
#include <qfont.h>
#include <qlistview.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qpopupmenu.h>
#include <qsplitter.h>
#include <qmessagebox.h>
#include "GTV.h"
#include "config.h"


void GTVMainWindow::OpenFile()
{
  QString s =
    QFileDialog::getOpenFileName(QString::null,"Game-tree files (*.gtf)",this);
  
  if ( !s.isEmpty() )
    {
      tree->openFile(s);        
    }
}


GTVMainWindow::GTVMainWindow( const char* config, const char* file )
{ 
  // Set initial window size.
  setGeometry(100,100,800,600);
  
  // Create program menus.
  QPopupMenu *menu;
  menu = new QPopupMenu(this);
  menu->insertItem("New");
  menu->insertItem("&Open",this, SLOT(OpenFile()));
  menu->insertSeparator();
  menu->insertItem("Exit", qApp, SLOT(quit()));
  menuBar()->insertItem("&File", menu);
  
  menu = new QPopupMenu(this);
  menu->insertItem("Contents");
  menu->insertSeparator();
  menu->insertItem("About");
  menuBar()->insertItem("&Help", menu);
  
  // The program main canvas
  QSplitter *splitterH = new QSplitter( QSplitter::Horizontal, this );
  QSplitter *splitterV = new QSplitter( QSplitter::Vertical, splitterH ); 

  tree  = new GameTreeView(splitterH, "GTViewer", config); 


  QString sboard;
  int x_cells  = 0;
  int y_cells  = 0;
  int x_size   = 0;
  int y_size   = 0;
  int x_start  = 0;
  int y_start  = 0;
  int x_offset = 0;
  int y_offset = 0;
  ifstream inCfg;

  QString name = config;
  name += "/gtv.cfg";

  inCfg.open(name.latin1());
  if ( inCfg ) {
    Config cfg(inCfg);
    string value;
    if ( cfg.getValue("BOARD_BACKGROUND", value) )
      sboard.sprintf("%s",value.c_str());
 
    if ( cfg.getValue("X_CELLS", value) )
      x_cells = atoi(value.c_str());
    if ( cfg.getValue("Y_CELLS", value) )
      y_cells = atoi(value.c_str());

    if ( cfg.getValue("X_SIZE", value) )
      x_size = atoi(value.c_str());
    if ( cfg.getValue("Y_SIZE", value) )
      y_size = atoi(value.c_str());

    if ( cfg.getValue("X_START", value) )
      x_start = atoi(value.c_str());
    if ( cfg.getValue("Y_START", value) )
      y_start = atoi(value.c_str());

    if ( cfg.getValue("X_OFFSET", value) )
      x_offset = atoi(value.c_str());
    if ( cfg.getValue("Y_OFFSET", value) )
      y_offset = atoi(value.c_str());
  }
  else
    cout << "Could not open config file:" << name << endl;
 

  board = new GameBoard(splitterV, "GTV", 
                        config, sboard,
			x_cells, y_cells,
			x_size, y_size,
			x_start, y_start,
			x_offset, y_offset ); 

  connect( tree, SIGNAL( currentChanged( QString ) ),
  	   board, SLOT( setBoard( QString ) ) );    

  setCentralWidget(splitterH);
  statusBar();

  QString str = file;
  if ( !str.isEmpty() )
    {
      tree->openFile(str);        
    }
}


GTVMainWindow::~GTVMainWindow( )
{
  delete board;
  delete tree;
}


int 
main( int argc, char **argv )
{
  QApplication a( argc, argv );
  GTVMainWindow w(argv[1],argv[2]);

  a.setMainWidget( &w );
  w.show();
  return a.exec();
}
