//
// GameTreeView
//
#include <iostream>
#include <dlfcn.h>
#include <stdlib.h>
#include "GameTreeView.h"
#include <qstring.h>
#include "posfp.h"


GameTreeViewItem::GameTreeViewItem(QListView *lv, GVFile *gvf,
			 const GVFile::NodeRecord& rec) : 
  QListViewItem(lv), m_gvf(gvf), m_rec(rec)
{
  setRecord(m_rec);
  setExpandable(m_rec.lc_id != 0);
}


GameTreeViewItem::GameTreeViewItem(GameTreeViewItem *gtvi, GVFile *gvf,
			 const GVFile::NodeRecord& rec) : 
  QListViewItem(gtvi), m_gvf(gvf), m_rec(rec)
{
  setRecord(m_rec);
  setExpandable(m_rec.lc_id != 0);
}

void GameTreeViewItem::setOpen( bool visible )
{
  // visible == visible
  if ( childCount() > 0 )  // isExpandable?
    {
      QListViewItem::setOpen(visible);
      return;
    }
  
  unsigned int no = m_rec.lc_id;
  while ( no != 0 )
    {
      GVFile::NodeRecord rec;
      if ( m_gvf->getRecord(no, rec) )
        {
	  new GameTreeViewItem(this, m_gvf, rec);
	  no = rec.ps_id;
        }
      else no = 0;
    }
  
  QListViewItem::setOpen(visible);
} 


int GameTreeViewItem::getMoves(Move moves[])
{
  GameTreeViewItem *parent = (GameTreeViewItem*) this->parent();
  int n = 0;

  if ( parent != 0 ) {
    n = parent->getMoves(moves);
    moves[n++] = m_rec.move;
  }
  
  return n;
} 



void GameTreeViewItem::getBoard(QString& str)
{
  Move       moves[400];  //Note
  char       str_pos[400]; //Note
  int        n;
  void      *start_pos,
            *end_pos;

  
  n = getMoves(moves);  //get depth

  str = m_gvf->getStartPos();

  // std::cout << str.latin1();
  //std::cout << endl;

  start_pos = pos_new();
  end_pos = pos_new();

  if ( start_pos != NULL && end_pos != NULL ) {
    pos_strToPos( str.latin1(), start_pos);
    pos_posToPos( start_pos, n, moves, end_pos, (void*)&m_rec.winlink);
    pos_posToStr( end_pos, str_pos);
  }
  
  pos_delete(end_pos);
  pos_delete(start_pos);

  str = str_pos;

  /*
  str.sprintf("In record number %d, %d ancestors.",
	      m_rec.id, n);
  */
} 


void GameTreeViewItem::setRecord(const GVFile::NodeRecord& rec)
{
  QString  s[8];
  char     str[256];
  void    *pos;

  pos = pos_new();
  if ( pos == NULL  )
    return;
  
  pos_moveToStr(pos, rec.move, str);
  s[0] = str;

  /*
  s[1].sprintf("%04d", (int)rec.lc_id );
  s[2].sprintf("%04d", (int)rec.ps_id );
  s[3].sprintf("%d",   (int)rec.alpha );
  s[4].sprintf("%d",   (int)rec.beta  );
  s[5].sprintf("%d",   (int)rec.eval  );
  pos_valueToStr(&pos, rec.alpha, str);
  s[3] = str;
  pos_valueToStr(&pos, rec.beta, str);
  s[4] = str;
  */
  
  
  pos_valueToStr(pos, rec.eval, str);
  s[1] = str;
  pos_moveToStr(pos, rec.bmove, str);
  s[2] = str;
  s[3].sprintf("%02d", (int)rec.height );
  s[4].sprintf("%02x", (int)rec.flags );
  s[5].sprintf("%04d", (int)rec.id    );
  sprintf(str, "%16llx", (unsigned long long) rec.winlink );
  s[6] = str;

  for ( int c=0; c<7 ; c++ )
    {
      setText(c, s[c]);
    }

  pos_delete(pos);
}


GameTreeView::GameTreeView(QWidget *parent, const char *name, 
			   QString config, bool sdo)
{
  const char *errmsg;
  QString path;

  path.sprintf("%s/libpos.so", config.latin1());

  handle = dlopen(path.latin1(), RTLD_NOW);
  if ( (errmsg=dlerror()) != NULL ) {
    fprintf(stderr, "Could not load %s (%s)\n", "./libpos.so", errmsg );
    exit(-1);
  }

  pos_init = (FPInit) dlsym(handle, "pos_init");
  if ( (errmsg=dlerror()) != NULL ) {
    fprintf(stderr, "Could not find %s (%s)\n", "pos_init", errmsg );
    exit(-1);
  }

  pos_new = (FPNew) dlsym(handle, "pos_new");
  if ( (errmsg=dlerror()) != NULL ) {
    fprintf(stderr, "Could not find %s (%s)\n", "pos_new", errmsg );
    exit(-1);
  }

  pos_delete = (FPDelete) dlsym(handle, "pos_delete");
  if ( (errmsg=dlerror()) != NULL ) {
    fprintf(stderr, "Could not find %s (%s)\n", "pos_delete", errmsg );
    exit(-1);
  }

  pos_moveToStr = (FPmoveToStr) dlsym(handle, "pos_moveToStr");
  if ( (errmsg=dlerror()) != NULL ) {
    fprintf(stderr, "Could not find %s (%s)\n", "pos_moveToStr", errmsg );
    exit(-1);
  }

  pos_valueToStr = (FPvalueToStr) dlsym(handle, "pos_valueToStr");
  if ( (errmsg=dlerror()) != NULL ) {
    fprintf(stderr, "Could not find %s (%s)\n", "pos_valueToStr", errmsg );
    exit(-1);
  }

  pos_posToStr = (FPposToStr) dlsym(handle, "pos_posToStr");
  if ( (errmsg=dlerror()) != NULL ) {
    fprintf(stderr, "Could not find %s (%s)\n", "pos_posToStr", errmsg );
    exit(-1);
  }

  pos_strToPos = (FPstrToPos) dlsym(handle, "pos_strToPos");
  if ( (errmsg=dlerror()) != NULL ) {
    fprintf(stderr, "Could not find %s (%s)\n", "pos_strToPos", errmsg );
    exit(-1);
  }

  pos_posToPos = (FPposToPos) dlsym(handle, "pos_posToPos");
  if ( (errmsg=dlerror()) != NULL ) {
    fprintf(stderr, "Could not find %s (%s)\n", "pos_posToPos", errmsg );
    exit(-1);
  }

  pos_init();


  lv = new QListView(parent, name, sdo);

  pGVF = 0;
  lv->addColumn( "Move" );
  //  lv->addColumn( "LC" );
  //lv->addColumn( "PS" );
  ///lv->addColumn( "A" );
  //lv->addColumn( "B" );
  lv->addColumn( "Eval" );
  lv->addColumn( "BMove" );
  lv->addColumn( "Height" );
  lv->addColumn( "Flags" );
  lv->addColumn( "Id" );
  lv->addColumn( "Link" );
  lv->setAllColumnsShowFocus( TRUE );
  lv->setSorting(-1);

  connect( lv, SIGNAL( currentChanged( QListViewItem * ) ),
	   this, SLOT( currentItemChanged( QListViewItem *) ) ); 
} 


GameTreeView::~GameTreeView() 
{
  delete lv;
  delete pGVF;

  dlclose(handle);
}


bool GameTreeView::openFile(QString filename)
{
  // Open file object (delete existing file object if one exists).
  delete pGVF;
  lv->clear(); // listview

  pGVF = new GVFile(filename);
  
  GVFile::NodeRecord rec;
  pGVF->getRootRecord(rec);

  new GameTreeViewItem(lv,pGVF,rec);
  lv->setSelected( lv->firstChild(),  TRUE );
  lv->setCurrentItem( lv->firstChild() );
  currentItemChanged( lv->firstChild() );
  return true;
}


void GameTreeView::currentItemChanged( QListViewItem *lvi )
{
  QString str;

  ((GameTreeViewItem*) lvi)->getBoard(str);

  emit currentChanged( str );
}
