#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "../pos.h"

#define SIZE  7

#define EMPTY 0
#define RED   1
#define BLUE  2
#define CONNECTION 3
#define RED_LAST   4
#define BLUE_LAST  5

#define BLUE_WIN 1235
#define RED_WIN  1236

#define getCol(move)   ((move)%SIZE)
#define getRow(move)   ((move)/SIZE)
#define getSq(col,row) ((row)*SIZE+(col))


typedef struct {
  int turn;
  int board[SIZE*SIZE];
} position_t;


static void
fen_posToStr(const position_t *pos, char str[])
{
  int r, c, n = 0;

  for ( r=0 ; r < SIZE ; r++ ) 
  {
      for ( c=0 ; c < SIZE ; c++ )
      {
          int sq = getSq(c,r);
          if ( pos->board[sq] == RED )
          {
              str[n++] = 'r';
          }
          else if ( pos->board[sq] == BLUE )
          {
              str[n++] = 'b';
          }
          else if ( pos->board[sq] == RED_LAST )
          {
              str[n++] = 'R';
          }
          else if ( pos->board[sq] == BLUE_LAST )
          {
              str[n++] = 'B';
          }
          else if ( pos->board[sq] == CONNECTION )
          {
              str[n++] = 'c';
          }
          else
          {
              str[n++] = '.';
          }
      }
  }
  str[n++] = ' ';
  str[n++] = (pos->turn == RED) ? 'r' : 'b';
  str[n] = '\0';
} 


static void
fen_strToPos(const char str[], position_t *pos)
{
    int sq, r, c, n=0;
    pos->turn = BLUE;
    for ( sq=0 ; sq<SIZE*SIZE ; sq++ )
        pos->board[sq] = EMPTY;
    
    for ( r=0 ; r < SIZE ; r++ ) 
    {
        for ( c=0 ; c < SIZE ; c++ )
        {
            int sq = getSq(c,r);
            
            if ( str[n] == 'r' )
            {
              pos->board[sq] = RED;
            }
            else if ( str[n] == 'R' )
            {
              pos->board[sq] = RED_LAST;
            }
            else if ( str[n] == 'b' )
            {
              pos->board[sq] = BLUE;
            }
            else if ( str[n] == 'B' )
            {
              pos->board[sq] = BLUE_LAST;
            }
            else if ( str[n] == 'c' )
            {
              pos->board[sq] = CONNECTION;
            }
            n++;
        }
    }
    
    if ( str[n] == ' ' )
    {
        n++;
        if ( str[n] == 'r' )
            pos->turn = RED;
        else if ( str[n] == 'b' )
            pos->turn = BLUE;
    }
} 


void
//
//
//
pos_init( )
{

}


void*
//
//
//
pos_new( )
{
  return (void*) malloc(sizeof(position_t));
}


void
//
//
//
pos_delete( void *vpos )
{
  free( (position_t*) vpos );
}


void
//
//
//
pos_moveToStr( const void *vpos, Move move, char str[] )
{
  char *cols = "abcdefghijklmnop";
  char *rows = "123456789ABCDEFG";

  // Mongoose uses ordering a1a2a3, whereas in here
  // a1b1c1 is used, thus switch col row.
  sprintf(str, "%c%c", cols[getRow(move)], rows[getCol(move)]);
}


void
//
//
//
pos_valueToStr( const void *vpos, Value value, char str[] )
{
  if ( value == BLUE_WIN )
    strcpy( str, "Blue");
  else if ( value == RED_WIN )
    strcpy( str, "Red");
  else
    sprintf(str, " %d ", value );
}

void
//
//
//
pos_posToStr( const void *vpos, char str[] )
{
  const position_t *pos = (position_t*) vpos;
  fen_posToStr(pos, str);
}


void
//
//
//
pos_strToPos( const char str[], void *vpos )
{
  position_t *pos = (position_t*) vpos;
  fen_strToPos( str,  pos);
}


void
//
//
//
pos_posToPos( const void *vstart, int nMoves, const Move move[],
              void *vend, void *vinfo )
{
  const position_t *start = (position_t*) vstart;
  position_t       *end   = (position_t*) vend;
  unsigned long long info = *((unsigned long long*)vinfo);
  position_t pos;
  int i;

  if ( start == NULL )
    {
      pos.turn = BLUE;
      for (  i=0 ; i<SIZE*SIZE ; i++ )
          pos.board[i] = EMPTY;
    }
  else
  {
      pos = *start;
  }
  for ( i=0 ; i<nMoves ; i++ )
  {
      // Mongoose uses ordering a1a2a3, whereas in here
      // a1b1c1 is used, thus switch col row.
      int m = getSq(getRow(move[i]),getCol(move[i]));
      pos.board[m] = pos.turn;
      if ( i == nMoves-1 )
      {
          if ( pos.board[m] == RED )
              pos.board[m] = RED_LAST;
          else
              pos.board[m] = BLUE_LAST;
      }
      pos.turn = (pos.turn == RED) ? BLUE : RED;
  }
  
  // virtual connection info.
  for ( i=0 ; i<SIZE*SIZE ; i++ ) 
  {
      if ( ((1ULL<<i) & info) != 0ULL )
      {
          int m = getSq(getRow(i),getCol(i));
          if ( pos.board[m] == EMPTY )
              pos.board[m] = CONNECTION;
      }
  }

  *end = pos;
}
