#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "gtf.h"


GTF* gtf_New(int max_depth)
{
  GTF *pGTF;

  pGTF = (GTF*) malloc(sizeof(GTF));
  if ( pGTF != NULL )
    {
      pGTF->stack = (GTFStack*) malloc(sizeof(GTFStack) * max_depth);
      if ( pGTF->stack != NULL ) 
	{
	  pGTF->d = 0;
	  pGTF->fp = NULL;
	  pGTF->id = 0;
	}
      else
	{
	  free(pGTF);
	  pGTF = NULL;
	}
    }
  
  return pGTF;
}


void  gtf_Free( GTF *pGTF )
{
  if ( pGTF != NULL )
    {
      free(pGTF->stack);
      free(pGTF);
    }
} 

 
void 
gtf_Start( GTF *pGTF, FILE *fp, const char *start_str )
{
  int d;

  pGTF->fp = fp;
  pGTF->d = d = 0;

  pGTF->stack[d].rec.id    = pGTF->id++;
  pGTF->stack[d].rec.lc_id = 0;
  pGTF->stack[d].rec.ps_id = 0;
  pGTF->stack[d].rec.alpha = 0;
  pGTF->stack[d].rec.beta  = 0;
  pGTF->stack[d].rec.eval  = 0;
  pGTF->stack[d].rec.move  = 0;
  pGTF->stack[d].rec.bmove = 0;
  write(fileno(pGTF->fp), start_str, strlen(start_str));
  write(fileno(pGTF->fp), "\n", 1);
  write(fileno(pGTF->fp), &pGTF->stack[d].rec, sizeof(NodeRecord));
}


void 
gtf_Stop( GTF *pGTF )
{
} 


void
gtf_NodeEnter(GTF *pGTF, int move, short alpha, short beta)
{
  int d = ++pGTF->d;

  // assert maxdepth 

  pGTF->stack[d].rec.id    = 0;
  pGTF->stack[d].rec.lc_id = 0;
  pGTF->stack[d].rec.ps_id = pGTF->stack[d-1].rec.lc_id;
  pGTF->stack[d].rec.alpha = alpha;
  pGTF->stack[d].rec.beta  = beta;
  pGTF->stack[d].rec.eval  = 0;
  pGTF->stack[d].rec.move  = move;
  pGTF->stack[d].rec.bmove = 0;;
}

void
gtf_NodeExit(GTF *pGTF, int bmove, short eval, short height, short flags,
             unsigned long long winlink )
{
  int d = pGTF->d--;

  pGTF->stack[d].rec.id      = pGTF->id++;
  pGTF->stack[d].rec.bmove   = bmove;
  pGTF->stack[d].rec.eval    = eval;
  pGTF->stack[d].rec.height  = height;
  pGTF->stack[d].rec.flags   = flags;
  pGTF->stack[d].rec.winlink = winlink;
  pGTF->stack[d-1].rec.lc_id = pGTF->stack[d].rec.id;
  write(fileno(pGTF->fp), &(pGTF->stack[d].rec), sizeof(NodeRecord));
}
